;/**************************************************************************//**
; * @file     startup_Z32F0641.s
; * @brief    CMSIS Cortex-M0 Core Device Startup File for
; *           Device Z32F0642 using IAR
; * @version  V3.10
; * @date     01. November 2016
; *
; * @note
; *
; ******************************************************************************/
;/* Copyright (c) 2012 ARM LIMITED
;
;   All rights reserved.
;   Redistribution and use in source and binary forms, with or without
;   modification, are permitted provided that the following conditions are met:
;   - Redistributions of source code must retain the above copyright
;     notice, this list of conditions and the following disclaimer.
;   - Redistributions in binary form must reproduce the above copyright
;     notice, this list of conditions and the following disclaimer in the
;     documentation and/or other materials provided with the distribution.
;   - Neither the name of ARM nor the names of its contributors may be used
;     to endorse or promote products derived from this software without
;     specific prior written permission.
;   *
;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
;   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
;   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
;   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
;   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
;   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
;   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
;   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
;   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
;   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
;   POSSIBILITY OF SUCH DAMAGE.
;   ---------------------------------------------------------------------------*/
;/*
;//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
;*/


        MODULE  ?cstartup

        ;; Forward declaration of sections.
        SECTION CSTACK:DATA:NOROOT(3)

        SECTION .intvec:CODE:NOROOT(2)

        EXTERN  __iar_program_start
        EXTERN  SystemInit
        PUBLIC  __vector_table
        PUBLIC	__vector_table_0x1C
        PUBLIC	__Vectors
        PUBLIC	__Vectors_End
        PUBLIC	__Vectors_Size

        DATA

__vector_table
		        DCD     sfe(CSTACK)              ; Top of Stack
                DCD     Reset_Handler             ; Reset Handler
                DCD     NMI_Handler               ; NMI Handler
                DCD     HardFault_Handler         ; Hard Fault Handler
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
__vector_table_0x1C              
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     SVC_Handler               ; SVCall Handler
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     PendSV_Handler            ; PendSV Handler
                DCD     SysTick_Handler           ; SysTick Handler

                ; External Interrupts
; ToDo:  Add here the vectors for the device specific external interrupts handler
	DCD		LVDDETECT_IRQHandler					;0
	DCD		SYSCLKFAIL_IRQHandler					;1
	DCD		MOSCFAIL_IRQHandler						;2
	DCD		SOSCFAIL_IRQHandler						;3
	DCD		WDT_IRQHandler							;4
	DCD		TIMER0_IRQHandler						;5
	DCD		TIMER1_IRQHandler						;6
	DCD		TIMER2_IRQHandler						;7
	DCD		TIMER3_IRQHandler						;8
	DCD		FRT_IRQHandler							;9
	DCD		GPIOAE_IRQHandler						;10
	DCD		GPIOAO_IRQHandler						;11
	DCD		GPIOBE_IRQHandler						;12
	DCD		GPIOBO_IRQHandler						;13
	DCD		GPIOCE_IRQHandler						;14
	DCD		GPIOCO_IRQHandler						;15
	DCD		GPIODE_IRQHandler						;16
	DCD		GPIODO_IRQHandler						;17
	DCD		MPWM_IRQHandler							;18
	DCD		MPWMPROT_IRQHandler						;19
	DCD		MPWMOVV_IRQHandler						;20
	DCD		I2C_IRQHandler							;21
	DCD		SPI_IRQHandler							;22
	DCD		UART0_IRQHandler						;23
	DCD		UART1_IRQHandler						;24
	DCD		ADC_IRQHandler							;25
	DCD		NULL_IRQHandler							;26
	DCD		NULL_IRQHandler							;27	
	DCD		NULL_IRQHandler							;28
	DCD		NULL_IRQHandler							;29
	DCD		NULL_IRQHandler							;30
	DCD		NULL_IRQHandler							;31
__Vectors_End
__Vectors		EQU		__vector_table
__Vectors_Size  EQU     __Vectors_End - __Vectors

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	
;;  Default interrupt handlers
;;
        THUMB

        PUBWEAK Reset_Handler
        SECTION .text:CODE:REORDER:NOROOT(2)
Reset_Handler

        LDR     R0, =SystemInit
        BLX     R0
        LDR     R0, =__iar_program_start
        BX      R0

        PUBWEAK NMI_Handler
        SECTION .text:CODE:REORDER:NOROOT(1)
NMI_Handler
        B NMI_Handler

        PUBWEAK HardFault_Handler
        SECTION .text:CODE:REORDER:NOROOT(1)
HardFault_Handler
        B HardFault_Handler

        PUBWEAK SVC_Handler
        SECTION .text:CODE:REORDER:NOROOT(1)
SVC_Handler
        B SVC_Handler

        PUBWEAK PendSV_Handler
        SECTION .text:CODE:REORDER:NOROOT(1)
PendSV_Handler
        B PendSV_Handler

        PUBWEAK SysTick_Handler
        SECTION .text:CODE:REORDER:NOROOT(1)
SysTick_Handler
        B SysTick_Handler

; Device Specific Handlers

;Null Handler - for Interrupts that don't exist
	PUBWEAK NULL_IRQHandler
        SECTION .text:CODE:REORDER:NOROOT(1)
NULL_IRQHandler 
	B NULL_IRQHandler

; External Interrupts

        PUBWEAK LVDDETECT_IRQHandler
        SECTION .text:CODE:REORDER:NOROOT(1)
LVDDETECT_IRQHandler
	B LVDDETECT_IRQHandler
		
	PUBWEAK SYSCLKFAIL_IRQHandler
        SECTION .text:CODE:REORDER:NOROOT(1)
SYSCLKFAIL_IRQHandler
	B SYSCLKFAIL_IRQHandler
		
	PUBWEAK MOSCFAIL_IRQHandler   		        
        SECTION .text:CODE:REORDER:NOROOT(1)
MOSCFAIL_IRQHandler
	B MOSCFAIL_IRQHandler

	PUBWEAK SOSCFAIL_IRQHandler   		        
        SECTION .text:CODE:REORDER:NOROOT(1)
SOSCFAIL_IRQHandler
	B SOSCFAIL_IRQHandler

		PUBWEAK WDT_IRQHandler   		
        SECTION .text:CODE:REORDER:NOROOT(1)
WDT_IRQHandler	
		B WDT_IRQHandler

		PUBWEAK FRT_IRQHandler   		
        SECTION .text:CODE:REORDER:NOROOT(1)
FRT_IRQHandler	
		B FRT_IRQHandler
		
		PUBWEAK TIMER0_IRQHandler      
        SECTION .text:CODE:REORDER:NOROOT(1)
TIMER0_IRQHandler	
 		B TIMER0_IRQHandler

		PUBWEAK TIMER1_IRQHandler      
		SECTION .text:CODE:REORDER:NOROOT(1)
TIMER1_IRQHandler	
  		B TIMER1_IRQHandler

		PUBWEAK TIMER2_IRQHandler      
		SECTION .text:CODE:REORDER:NOROOT(1)
TIMER2_IRQHandler	
 		B TIMER2_IRQHandler
			
		PUBWEAK TIMER3_IRQHandler      
		SECTION .text:CODE:REORDER:NOROOT(1)
TIMER3_IRQHandler	
  		B TIMER3_IRQHandler
		
       PUBWEAK GPIOAE_IRQHandler      
       SECTION .text:CODE:REORDER:NOROOT(1)
GPIOAE_IRQHandler	
 		B GPIOAE_IRQHandler
			
		PUBWEAK GPIOAO_IRQHandler      
        SECTION .text:CODE:REORDER:NOROOT(1)
GPIOAO_IRQHandler	
 		B GPIOAO_IRQHandler
			
		PUBWEAK GPIOBE_IRQHandler      
	    SECTION .text:CODE:REORDER:NOROOT(1)
GPIOBE_IRQHandler	
  		B GPIOBE_IRQHandler
			
		PUBWEAK GPIOBO_IRQHandler      
        SECTION .text:CODE:REORDER:NOROOT(1)
GPIOBO_IRQHandler	
 		B GPIOBO_IRQHandler
			
		PUBWEAK GPIOCE_IRQHandler      
        SECTION .text:CODE:REORDER:NOROOT(1)
GPIOCE_IRQHandler	
 		B GPIOCE_IRQHandler
			
		PUBWEAK GPIOCO_IRQHandler      
        SECTION .text:CODE:REORDER:NOROOT(1)
GPIOCO_IRQHandler	
		B GPIOCO_IRQHandler
			
		PUBWEAK GPIODE_IRQHandler      
        SECTION .text:CODE:REORDER:NOROOT(1)
GPIODE_IRQHandler	
		B GPIODE_IRQHandler
			
		PUBWEAK GPIODO_IRQHandler      
        SECTION .text:CODE:REORDER:NOROOT(1)
GPIODO_IRQHandler	
		B GPIODO_IRQHandler
			
		PUBWEAK MPWM_IRQHandler       
        SECTION .text:CODE:REORDER:NOROOT(1)
MPWM_IRQHandler	
		B MPWM_IRQHandler

		PUBWEAK MPWMPROT_IRQHandler   
        SECTION .text:CODE:REORDER:NOROOT(1)
MPWMPROT_IRQHandler
		B MPWMPROT_IRQHandler

		PUBWEAK MPWMOVV_IRQHandler  
        SECTION .text:CODE:REORDER:NOROOT(1)
MPWMOVV_IRQHandler
		B MPWMOVV_IRQHandler

		PUBWEAK SPI_IRQHandler   			
        SECTION .text:CODE:REORDER:NOROOT(1)
SPI_IRQHandler	
		B SPI_IRQHandler
		
		PUBWEAK I2C_IRQHandler        
        SECTION .text:CODE:REORDER:NOROOT(1)
I2C_IRQHandler	
		B I2C_IRQHandler	
			
		PUBWEAK UART0_IRQHandler       
        SECTION .text:CODE:REORDER:NOROOT(1)
UART0_IRQHandler	
		B UART0_IRQHandler
			
		PUBWEAK UART1_IRQHandler       
        SECTION .text:CODE:REORDER:NOROOT(1)
UART1_IRQHandler	
		B UART1_IRQHandler
			
		PUBWEAK ADC_IRQHandler       
        SECTION .text:CODE:REORDER:NOROOT(1)
ADC_IRQHandler	
		B ADC_IRQHandler

	END	
                
;/*************  END OF FILE  **************************************/
