/* File: startup_Z32F0642.S
 * Purpose:  CMSIS Cortex-M0 Core Device Startup File for
 *          Device Z32F0642 for GCC
 * Version: V1.0
 * Date: 01 November 2016
 *
/* Copyright (c) 2011 - 2013 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

	.syntax	unified
	.cpu cortex-m0
	.thumb

	.section .stack
	.align	3
#ifdef __STACK_SIZE
	.equ	Stack_Size, __STACK_SIZE
#else
	.equ	Stack_Size, 0x00000200
#endif
	.globl	__StackTop
	.globl	__StackLimit
__StackLimit:
	.space	Stack_Size
	.size	__StackLimit, . - __StackLimit
__StackTop:
	.size	__StackTop, . - __StackTop

	.section .heap
	.align	3
#ifdef __HEAP_SIZE
	.equ	Heap_Size, __HEAP_SIZE
#else
	.equ	Heap_Size, 0x00000000
#endif
	.globl	__HeapBase
	.globl	__HeapLimit
__HeapBase:
	.if	Heap_Size
	.space	Heap_Size
	.endif
	.size	__HeapBase, . - __HeapBase
__HeapLimit:
	.size	__HeapLimit, . - __HeapLimit


	.section .isr_vector
	.align	2
	.globl	__isr_vector

__isr_vector:
	.word	__StackTop            /* Top of Stack */
	.word	Reset_Handler         /* Reset Handler */
	.word	NMI_Handler           /* NMI Handler */
	.word	HardFault_Handler     /* Hard Fault Handler */
	.word	0                     /* Reserved */
	.word	0                     /* Reserved */
	.word	0                     /* Reserved */
	.word	0                     /* Reserved */
	.word	0                     /* Reserved */
	.word	0                     /* Reserved */
	.word	0                     /* Reserved */
	.word	SVC_Handler           /* SVCall Handler */
	.word	0                     /* Reserved */
	.word	0                     /* Reserved */
	.word	PendSV_Handler        /* PendSV Handler */
	.word	SysTick_Handler       /* SysTick Handler */

	/* External interrupts */

	.word		LVDDETECT_IRQHandler						
	.word		SYSCLKFAIL_IRQHandler						
	.word		MOSCFAIL_IRQHandler						
	.word		SOSCFAIL_IRQHandler						
	.word		WDT_IRQHandler							
	.word		TIMER0_IRQHandler						
	.word		TIMER1_IRQHandler						
	.word		TIMER2_IRQHandler						
	.word		TIMER3_IRQHandler
	.word		FRT_IRQHandler						
	.word		GPIOAE_IRQHandler						
	.word		GPIOAO_IRQHandler						
	.word		GPIOBE_IRQHandler						
	.word		GPIOBO_IRQHandler						
	.word		GPIOCE_IRQHandler						
	.word		GPIOCO_IRQHandler						
	.word		GPIODE_IRQHandler						
	.word		GPIODO_IRQHandler						
	.word		MPWM_IRQHandler						
	.word		MPWMPROT_IRQHandler					
	.word		MPWMOVV_IRQHandler						
	.word		I2C_IRQHandler							
	.word		SPI_IRQHandler							
	.word		UART0_IRQHandler						
	.word		UART1_IRQHandler						
	.word		ADC_IRQHandler							
	.word		NULL_IRQHandler							
	.word		NULL_IRQHandler							
	.word		NULL_IRQHandler							
	.word		NULL_IRQHandler							
	.word		NULL_IRQHandler							
	.word		NULL_IRQHandler						

	.size	__isr_vector, . - __isr_vector

	.text
	.thumb
	.thumb_func
	.align	2
	.globl	Reset_Handler
	.type	Reset_Handler, %function
Reset_Handler:

/*  Firstly it copies data from read only memory to RAM. There are two schemes
 *  to copy. One can copy more than one sections. Another can only copy
 *  one section.  The former scheme needs more instructions and read-only
 *  data to implement than the latter.
 *  Macro __STARTUP_COPY_MULTIPLE is used to choose between two schemes.  */

#ifdef __STARTUP_COPY_MULTIPLE
/*  Multiple sections scheme.
 *
 *  Between symbol address __copy_table_start__ and __copy_table_end__,
 *  there are array of triplets, each of which specify:
 *    offset 0: LMA of start of a section to copy from
 *    offset 4: VMA of start of a section to copy to
 *    offset 8: size of the section to copy. Must be multiply of 4
 *
 *  All addresses must be aligned to 4 bytes boundary.
 */
	ldr	r4, =__copy_table_start__
	ldr	r5, =__copy_table_end__

.L_loop0:
	cmp	r4, r5
	bge	.L_loop0_done		/* no more data */

; otherwise, load the data information:	
	
	ldr	r1, [r4]			/* start of the array (flash address pointer)  */
	ldr	r0, [r4, #4]		/* start of the data (ram)  */
	ldr	r2, [r4, #8]		/* size of the data (ram)  */
	adds r2,r0,r2			/* set r2 to end address for simple loop  */
	
; Now just handle as the simple:

	cmp r0,r2				/* is there data to copy?data pos == data end?  */
	bge	copydata_done		/* data pos is greater than or equal to end, so we are done  */

.copydata_check:
	;load and store the data
	ldr r3,[r1]				/* load the ram value (4 bytes)  */
	str r3,[r0]				/* store in ram  */
	adds r0,r0,#4			/* move to the data pointer  */
	cmp r0,r2
	ble	copydata_check		/* ok, just loop now   */
	adds r1,r1,#4			/* move the flash pointer   */

.copydata_done:		

; Do next item
	adds r4,r4,#12			/* position to the beginninng of next erray   */
	b L_loop0				/* do it again   */

.L_loop0_done:
#else
/*  Single section scheme.
 *
 *  The ranges of copy from/to are specified by following symbols
 *    __etext: LMA of start of the section to copy from. Usually end of text
 *    __data_start__: VMA of start of the section to copy to
 *    __data_end__: VMA of end of the section to copy to
 *
 *  All addresses must be aligned to 4 bytes boundary.
 */

	ldr	r1, =__etext        /* start address in flash, will be on 4byte boundry already(see ld script)  */
	ldr r2,	=__data_end__	/* load the end data address in ram for comparison  */
	
	ldr	r0, =__data_start__ /* starting ram address, data(ram) position  */

	cmp r0,r2				/* is there data to copy?data pos == data end?  */
	bge	.copydata_done		/* data pos is greater than or equal to end, so we are done  */

.copydata_check:
	/*  load and store the data */
	
	ldr r3,[r1]				/* load the ram value (4 bytes)   */
	str r3,[r0]				/* store in ram   */
	adds r0,r0,#4			/* move to the data pointer  */
	adds r1,r1,#4			/* move the flash pointer   */
	cmp r0,r2
	ble	.copydata_check		/* ok, just loop now  */

.copydata_done:		

#endif /*__STARTUP_COPY_MULTIPLE */

/*  This part of work usually is done in C library startup code. Otherwise,
 *  define this macro to enable it in this startup.
 *
 *  There are two schemes too. One can clear multiple BSS sections. Another
 *  can only clear one section. The former is more size expensive than the
 *  latter.
 *
 *  Define macro __STARTUP_CLEAR_BSS_MULTIPLE to choose the former.
 *  Otherwise efine macro __STARTUP_CLEAR_BSS to choose the later.
 */
#ifdef __STARTUP_CLEAR_BSS_MULTIPLE
/*  Multiple sections scheme.
 *
 *  Between symbol address __copy_table_start__ and __copy_table_end__,
 *  there are array of tuples specifying:
 *    offset 0: Start of a BSS section
 *    offset 4: Size of this BSS section. Must be multiply of 4
 */
	ldr	r3, =__zero_table_start__
	ldr	r4, =__zero_table_end__

.L_loop1:
	cmp	r3, r4
	bge	.L_loop1_done
	ldr	r1, [r3]
	ldr	r2, [r3, #4]    /* size  */
	add r2,r2,r1		/* r2 now points to the end  */
	movs	r0, 0

.zeroloop:
	cmp	r1, r2		/* are we done?  */
	bge .zeroloop_done	/* done!  */
	str r0,[r1];
	adds r1,r1,#4
	b .zeroloop
	
.zeroloop_done:
 ;get next array values
 adds r3,r3,#8				/* set to the next array  */
 b .L_loop1					/* get the next data  */
 
.L_loop1_done:
#elif defined (__STARTUP_CLEAR_BSS)
/*  Single BSS section scheme.
 *
 *  The BSS section is specified by following symbols
 *    __bss_start__: start of the BSS section.
 *    __bss_end__: end of the BSS section.
 *
 *  Both addresses must be aligned to 4 bytes boundary.
 */
	ldr	r1, =__bss_start__
	ldr	r2, =__bss_end__

	movs	r0, 0	/* zero ram  */
.zeroloop:
	cmp	r1, r2		/* are we done?  */
	bge .zeroloop_done	/* done!  */
	str r0,[r1];
	adds r1,r1,#4
	b .zeroloop
	
.zeroloop_done:

#endif /* __STARTUP_CLEAR_BSS_MULTIPLE || __STARTUP_CLEAR_BSS */

	bl	SystemInit
	bl	__START

	.pool
	.size	Reset_Handler, . - Reset_Handler

/*  end of reset handler  */

	.align	2
	.thumb_func
	.weak	Default_Handler
	.type	Default_Handler, %function
Default_Handler:
	b	.
	.size	Default_Handler, . - Default_Handler

/*    Macro to define default handlers. Default handler
 *    will be weak symbol and just dead loops. They can be
 *    overwritten by other handlers 
 */
	.macro	def_irq_handler	handler_name
	.weak	\handler_name
	.set	\handler_name, Default_Handler
	.endm


	def_irq_handler NULL_IRQHandler		/* IRQ handler for where there are no irqs being used */


/*	Assign all handlers to weak default function */

	def_irq_handler	NMI_Handler
	def_irq_handler	HardFault_Handler
	def_irq_handler	SVC_Handler
	def_irq_handler	PendSV_Handler
	def_irq_handler	SysTick_Handler
	def_irq_handler	LVDDETECT_IRQHandler    
	def_irq_handler	SYSCLKFAIL_IRQHandler
	def_irq_handler	MOSCFAIL_IRQHandler   
	def_irq_handler	SOSCFAIL_IRQHandler   
	def_irq_handler	WDT_IRQHandler         
	def_irq_handler	FRT_IRQHandler         
	def_irq_handler	TIMER0_IRQHandler      
	def_irq_handler	TIMER1_IRQHandler      
	def_irq_handler	TIMER2_IRQHandler      
	def_irq_handler	TIMER3_IRQHandler      
	def_irq_handler	GPIOAE_IRQHandler      
	def_irq_handler	GPIOAO_IRQHandler        
	def_irq_handler	GPIOBE_IRQHandler      
	def_irq_handler	GPIOBO_IRQHandler        
	def_irq_handler	GPIOCE_IRQHandler      
	def_irq_handler	GPIOCO_IRQHandler        
	def_irq_handler	GPIODE_IRQHandler      
	def_irq_handler	GPIODO_IRQHandler
	def_irq_handler	MPWM_IRQHandler       
	def_irq_handler	MPWMPROT_IRQHandler   
	def_irq_handler	MPWMOVV_IRQHandler  
	def_irq_handler	SPI_IRQHandler        
	def_irq_handler	I2C_IRQHandler        
	def_irq_handler	UART0_IRQHandler       
	def_irq_handler	UART1_IRQHandler       
	def_irq_handler	ADC_IRQHandler        

	.end
